{*******************************************************}
{                                                       }
{          MiTeC Enhanced Button Group Component        }
{                   version 1.3                         }
{                                                       }
{            Copyright (c) 2007-2012 Michal Mutl        }
{                                                       }
{*******************************************************}

unit MiTeC_ButtonGroup;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, CommCtrl, CategoryButtons, GraphUtil, ButtonGroup, Themes;

const
  cAbout = 'Enhanced ButtonGroup Control 1.3 - Copyright  2008-2012 Michal Mutl';

type
  PNotifyEvent = ^TNotifyEvent;

  TButtonGroupEx = class(TButtonGroup)
  private
    FAbout: string;
    FOnBeforeDrawButton: TGrpButtonDrawEvent;
    FOnAfterDrawButton: TGrpButtonDrawEvent;
    FOnDrawButton: TGrpButtonDrawEvent;
    FOnDrawIcon: TGrpButtonDrawIconEvent;
    FOnDrawText: TGrpButtonDrawEvent;
    FRegularButtonColor: TColor;
    FSelectedButtonColor: TColor;
    FHotButtonColor: TColor;
    FHotButton: Integer;
    FTextIndent: Byte;
    FEdgeButtonColor: TColor;
    FGlassButtonStyle: Boolean;
    FGlassButtonColor: TColor;
    FBackgroundGradientColor: TColor;
    FBackGradientDirection: TGradientDirection;
    FBorderColor: TColor;
    FWW: Boolean;
    FAutoSize: boolean;
    procedure SetOnDrawButton(const Value: TGrpButtonDrawEvent);
    procedure SetOnDrawIcon(const Value: TGrpButtonDrawIconEvent);
    procedure SetHotButtonColor(const Value: TColor);
    procedure SetRegularButtonColor(const Value: TColor);
    procedure SetSelectedButtonColor(const Value: TColor);
    procedure SetTextIndent(const Value: Byte);
    procedure SetEdgeButtonColor(const Value: TColor);
    procedure SetGlassButtonColor(const Value: TColor);
    procedure SetGlassButtonStyle(const Value: Boolean);
    procedure SetBackGradientDirection(const Value: TGradientDirection);
    procedure SetBackgroundGradientColor(const Value: TColor);
    procedure DoFillRect(const Rect: TRect);
    procedure SetBorderColor(const Value: TColor);
    procedure SetWW(const Value: Boolean);
    procedure SetAutoSize(const Value: boolean);
  protected
    function CalcButtonsPerRow: Integer;
    function CalcRowsSeen: Integer;
    procedure DrawButton(Index: Integer; Canvas: TCanvas; Rect: TRect; State: TButtonDrawState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer); override;
    procedure Paint; override;
    procedure Resize; override;
    procedure DoItemClicked(const Index: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property HotButtonIndex: Integer read FHotButton;
  published
    property AutoSize: boolean read FAutoSize write SetAutoSize;
    property About: string read FAbout;
    //property BackgroundGradientColor: TColor read FBackgroundGradientColor write SetBackgroundGradientColor default clNone;
    //property BackgroundGradientDirection: TGradientDirection read FBackGradientDirection write SetBackGradientDirection default gdHorizontal;
    property Color;
    property GlassButtonStyle: Boolean read FGlassButtonStyle write SetGlassButtonStyle default False;
    property GlassButtonColor: TColor read FGlassButtonColor write SetGlassButtonColor default clWhite;
    property EdgeButtonColor: TColor read FEdgeButtonColor write SetEdgeButtonColor default clBtnFace;
    //property BorderColor: TColor read FBorderColor write SetBorderColor default clGray;
    property HotButtonColor: TColor read FHotButtonColor write SetHotButtonColor default clMedGray;
    property RegularButtonColor: TColor read FRegularButtonColor write SetRegularButtonColor nodefault;
    property SelectedButtonColor: TColor read FSelectedButtonColor write SetSelectedButtonColor nodefault;
    property TextIndent: Byte read FTextIndent write SetTextIndent;
    property WordWrap: Boolean read FWW write SetWW;
    property OnAfterDrawButton: TGrpButtonDrawEvent read FOnAfterDrawButton write FOnAfterDrawButton;
    property OnBeforeDrawButton: TGrpButtonDrawEvent read FOnBeforeDrawButton write FOnBeforeDrawButton;
    property OnDrawButton: TGrpButtonDrawEvent read FOnDrawButton write SetOnDrawButton;
    property OnDrawIcon: TGrpButtonDrawIconEvent read FOnDrawIcon write SetOnDrawIcon;
    property OnDrawText: TGrpButtonDrawEvent read FOnDrawText write FOnDrawText;
  end;

function GetButtonCaption(ACaption: string): string;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MiTeC', [TButtonGroupEx]);
end;

function GetButtonCaption;
var
  p: Integer;
begin
  p:=Pos(#9,ACaption);
  if p>0 then
    Result:=Copy(ACaption,1,p-1)
  else
    Result:=ACaption;  
end;

procedure DrawBevel(ACanvas: TCanvas; ALeft, ATop, ALength: integer; ATopColor: TColor = clBlack; ABottomColor: TColor = clWhite);
var
  pc: TColor;
begin
  with ACanvas do begin
    pc:=Pen.Color;
    Pen.Color:=ATopColor;
    MoveTo(ALeft,ATop);
    LineTo(ALeft+ALength,ATop);
    MoveTo(ALeft,ATop+1);
    Pen.Color:=ABottomColor;
    LineTo(ALeft+ALength,ATop+1);
    Pen.Color:=pc;
  end;
end;

function BlendColor(UpColor, DownColor: TColor; Amount: Integer): TColor;
var
  Value: Cardinal;
begin
  Assert(Amount in [0..255]);
  Value := Amount xor 255;
  if Integer(UpColor) < 0 then UpColor := GetSysColor(UpColor and $000000FF);
  if Integer(DownColor) < 0 then DownColor := GetSysColor(DownColor and $000000FF);
  Result := Integer(
    ((Cardinal(UpColor) and $FF00FF) * Cardinal(Amount) +
    (Cardinal(DownColor) and $FF00FF) * Value) and $FF00FF00 +
    ((Cardinal(UpColor) and $00FF00) * Cardinal(Amount) +
    (Cardinal(DownColor) and $00FF00) * Value) and $00FF0000) shr 8;
end;

procedure PutColorValues24(var P: PByte; r, g, b: Byte);
begin
  P^ := b;
  Inc(P);
  P^ := g;
  Inc(P);
  P^ := r;
  Inc(P);
end;

procedure DrawVertGlass(Canvas: TCanvas; ARect: TRect; FromColor, ToColor: TColor;
  GlowHeight: Integer);
var
  Bitmap: TBitmap;
  i, j: Integer;
  c: TColor;
  P: PByte;
  am, am2: Integer;
begin
  Bitmap := TBitmap.Create;
  Bitmap.Width := ARect.Right - ARect.Left;
  Bitmap.Height := ARect.Bottom - ARect.Top;
  Bitmap.Canvas.CopyRect(Rect(0, 0, Bitmap.Width, Bitmap.Height), Canvas, Rect(ARect.Left, ARect.Top, ARect.Right, ARect.Bottom));
  Bitmap.PixelFormat := pf24bit;

  for i := 0 to Bitmap.Height - 1 do
  begin
    am := Round(255 * (i / Bitmap.Height));
    C := BlendColor(ToColor, FromColor, am);

    if i < GlowHeight then
    begin
      am2 := Round(255 * (i / (GlowHeight * 2)));
      C := BlendColor(C, clWhite, am2);
    end;

  	P := Bitmap.ScanLine[i];
    for j := 0 to Bitmap.Width -1 do
    begin
      PutColorValues24(P, GetRValue(c), GetGValue(c), GetBValue(c));
	  end;
  end;
  //Bitmap.SaveToFile('c:\0\test.bmp');
  Canvas.Draw(ARect.Left, ARect.Top, Bitmap);
  FreeAndNil(Bitmap);
end;

{ TButtonGroupEx }

function TButtonGroupEx.CalcButtonsPerRow: Integer;
begin
  if gboFullSize in ButtonOptions then
    Result:=1
  else begin
    Result:=ClientWidth div ButtonWidth;
    if Result=0 then
      Result:=1;
  end;
end;

function TButtonGroupEx.CalcRowsSeen: Integer;
begin
  Result:=ClientHeight div ButtonHeight
end;

constructor TButtonGroupEx.Create(AOwner: TComponent);
begin
  FAbout:=cAbout;

  inherited;

  FGlassButtonStyle:=False;
  FHotButtonColor:=$00EFD3C6;
  FSelectedButtonColor:=clInfoBk;
  FEdgeButtonColor:=clMedGray;
  FGlassButtonColor:=clWhite;
  FRegularButtonColor:=GetHighlightColor(clBtnFace, 15);
  FBackgroundGradientColor:=clNone;
  FBackGradientDirection:=gdHorizontal;
  FTextIndent:=5;
end;

destructor TButtonGroupEx.Destroy;
begin
  inherited;
end;


procedure TButtonGroupEx.DoFillRect(const Rect: TRect);
begin
  if ParentBackground and ThemeServices.ThemesEnabled then
    ThemeServices.DrawParentBackground(Handle, Canvas.Handle, nil, False, @Rect)
  else
    Canvas.FillRect(Rect);
end;

procedure TButtonGroupEx.DoItemClicked(const Index: Integer);
var
  LButton: TGrpButtonItem;
  LOnClick: TNotifyEvent;
begin
  ItemIndex:=Index;
  inherited;
end;

procedure TButtonGroupEx.DrawButton(Index: Integer; Canvas: TCanvas;
  Rect: TRect; State: TButtonDrawState);
var
  TextLeft, TextTop, TextH: Integer;
  RectHeight,p: Integer;
  ImgTop: Integer;
  TextOffset: Integer;
  FillColor: TColor;
  EdgeColor: TColor;
  InsertIndication: TRect;
  TextRect: TRect;
  OrgRect: TRect;
  s,Caption,SubCaption1,SubCaption2: string;
  TextFormat: TTextFormat;
begin
  if Assigned(FOnDrawButton) and (not (csDesigning in ComponentState)) then
    FOnDrawButton(Self, Index, Canvas, Rect, State)
  else begin
    OrgRect := Rect;
    if Assigned(FOnBeforeDrawButton) then
      FOnBeforeDrawButton(Self, Index, Canvas, Rect, State);
    InflateRect(Rect, -1, -1);

    Canvas.Font.Assign(Self.Font);
    if bdsHot in State then begin
      FillColor := FHotButtonColor;
      if bdsSelected in State then
        FillColor := GetShadowColor(FillColor,0);
      EdgeColor := GetShadowColor(FillColor);
    end else if bdsSelected in State then begin
      FillColor := FSelectedButtonColor;
      EdgeColor := GetShadowColor(FillColor);
    end else begin
      FillColor := FRegularButtonColor;
      if (bdsFocused in State) then
        EdgeColor := GetShadowColor(FillColor)
      else
        EdgeColor := GetShadowColor(FillColor);
    end;
    if EdgeButtonColor<>clNone then
      EdgeColor:=EdgeButtonColor;

    Canvas.Brush.Color := FillColor;
    if FillColor <> clNone then begin
      if not GlassButtonStyle then
        Canvas.FillRect(Rect)
      else
        DrawVertGlass(Canvas, Rect, GlassButtonColor, FillColor, (Rect.Bottom-Rect.Top) div 2);

      { Draw the edge outline }
      Canvas.Brush.Color := EdgeColor;
      Canvas.FrameRect(Rect);
    end;

    Canvas.Brush.Color := FillColor;

    { Compute the text location }
    s:=Items[Index].Caption;
    Caption:=s;
    SubCaption1:='';
    SubCaption2:='';
    p:=Pos(#9,s);
    if p>0 then begin
      Caption:=Copy(s,1,p-1);
      Delete(s,1,p);
      p:=Pos(#9,s);
      if p>0 then begin
        SubCaption1:=Copy(s,1,p-1);
        Delete(s,1,p);
        SubCaption2:=s;
      end else
        SubCaption1:=s;
    end;

    TextLeft := Rect.Left;
    RectHeight := Rect.Bottom - Rect.Top;
    TextRect.Left := TextLeft + FTextIndent;
    TextRect.Right := Rect.Right-2;
    TextRect.Top := Rect.Top;
    TextRect.Bottom := Rect.Bottom-1;
    TextLeft := Rect.Left + 4;
    TextH:=Canvas.TextHeight('Wg');
    if SubCaption2<>'' then
      TextTop := Rect.Top + (RectHeight - 3*TextH) div 2
    else if SubCaption1<>'' then
      TextTop := Rect.Top + (RectHeight - 2*TextH) div 2
    else begin
      if FWW then
        TextH:=DrawText(Canvas.Handle,PChar(Caption),Length(Caption),TextRect,DT_WORDBREAK or DT_CALCRECT);
      TextTop:=Rect.Top+(RectHeight-TextH) div 2;
    end;

    if gboFullSize in ButtonOptions then
      Inc(TextLeft, 4);  // indent, slightly

    if TextTop < Rect.Top then
      TextTop := Rect.Top;
    if bdsDown in State then begin
      Inc(TextTop);
      Inc(TextLeft);
    end;

    { Draw the icon - prefer the event }
    TextOffset := 0;
    if Assigned(FOnDrawIcon) then
      FOnDrawIcon(Self, Index, Canvas, OrgRect, State, TextOffset)
    else if (Images <> nil) and (Items[Index].ImageIndex > -1) and
        (Items[Index].ImageIndex < Images.Count) then begin
      ImgTop := Rect.Top + (RectHeight - Images.Height) div 2;
      if ImgTop < Rect.Top then
        ImgTop := Rect.Top;
      if bdsDown in State then
        Inc(ImgTop);
      Images.Draw(Canvas, TextLeft - 1, ImgTop, Items[Index].ImageIndex);
      TextOffset := Images.Width + 1;
    end;

    { Show insert indications }
    if [bdsInsertLeft, bdsInsertTop, bdsInsertRight, bdsInsertBottom] * State <> [] then begin
      Canvas.Brush.Color := GetShadowColor(EdgeColor);
      InsertIndication := Rect;
      if bdsInsertLeft in State then begin
        Dec(InsertIndication.Left, 2);
        InsertIndication.Right := InsertIndication.Left + 2;
      end else if bdsInsertTop in State then begin
        Dec(InsertIndication.Top);
        InsertIndication.Bottom := InsertIndication.Top + 2;
      end else if bdsInsertRight in State then begin
        Inc(InsertIndication.Right, 2);
        InsertIndication.Left := InsertIndication.Right - 2;
      end else if bdsInsertBottom in State then begin
        Inc(InsertIndication.Bottom);
        InsertIndication.Top := InsertIndication.Bottom - 2;
      end;
      Canvas.FillRect(InsertIndication);
      Canvas.Brush.Color := FillColor;
    end;

    if gboShowCaptions in ButtonOptions then begin
      //if FillColor = clNone then
        Canvas.Brush.Style := bsClear;

      // Avoid clipping the image
      Inc(TextLeft, TextOffset);
      TextRect.Left := TextLeft + FTextIndent;
      TextRect.Right := Rect.Right - 2;
      TextRect.Top := TextTop;
      TextRect.Bottom := Rect.Bottom -1;

      if Assigned(FOnDrawText) then
        FOnDrawText(Self, Index, Canvas, TextRect, State)
      else begin
        if FWW and (SubCaption1+SubCaption2='') then
          TextFormat:=[tfWordBreak]
        else
          TextFormat:=[tfEndEllipsis];
        Canvas.TextRect(TextRect, Caption, TextFormat);
        TextRect.Top:=TextRect.Top+Canvas.TextHeight('Wg')+3;
        Canvas.Font.Style:=[];
        if FWW then
          TextFormat:=[tfWordBreak]
        else
          TextFormat:=[tfEndEllipsis];
        Canvas.TextRect(TextRect, SubCaption1, TextFormat);
        TextRect.Top:=TextRect.Top+Canvas.TextHeight('Wg');
        Canvas.TextRect(TextRect, SubCaption2, TextFormat);
      end;
    end;

    if Assigned(FOnAfterDrawButton) then
      FOnAfterDrawButton(Self, Index, Canvas, OrgRect, State);
  end;
  Canvas.Brush.Color := Color; { Restore the original color }
end;

procedure TButtonGroupEx.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited;
  FHotButton:=Self.IndexOfButtonAt(X,Y);
end;

procedure TButtonGroupEx.Paint;
begin
  Canvas.Brush.Color := Color;
  if FBackgroundGradientColor = clNone then
    DoFillRect(ClientRect)
  else
    GradientFillCanvas(Canvas, Color, FBackgroundGradientColor, Canvas.ClipRect, FBackGradientDirection);
  if Items.Count > 0 then
    inherited;
end;

procedure TButtonGroupEx.Resize;
var
  RowsSeen: Integer;
  ButtonsPerRow: Integer;
  TotalRowsNeeded: Integer;
  BevelHeight: Integer;
begin
  inherited;

  if FAutoSize then begin
    RowsSeen:=CalcRowsSeen;
    ButtonsPerRow:=CalcButtonsPerRow;

    BevelHeight:=2;
    if BorderStyle<>bsNone then
      Inc(BevelHeight,4)
    else
      if BevelKind<>bkNone then begin
        if beTop in BevelEdges then
          Inc(BevelHeight,2);
        if beBottom in BevelEdges then
          Inc(BevelHeight,2);
      end;

    Windows.ShowScrollBar(Handle,SB_VERT,False);
    TotalRowsNeeded:=Items.Count div ButtonsPerRow;
    if Items.Count mod ButtonsPerRow<>0 then
      Inc(TotalRowsNeeded);
    Height:=TotalRowsNeeded*ButtonHeight+BevelHeight;
  end;
end;

procedure TButtonGroupEx.SetAutoSize(const Value: boolean);
begin
  if FAutoSize<>Value then begin
    FAutoSize:=Value;
    Resize;
    Invalidate;
  end;
end;

procedure TButtonGroupEx.SetBackGradientDirection(
  const Value: TGradientDirection);
begin
  if FBackGradientDirection <> Value then begin
    FBackGradientDirection := Value;
    Invalidate;
  end;
end;

procedure TButtonGroupEx.SetBackgroundGradientColor(const Value: TColor);
begin
  if FBackgroundGradientColor <> Value then begin
    FBackgroundGradientColor := Value;
    Invalidate;
  end;
end;

procedure TButtonGroupEx.SetBorderColor(const Value: TColor);
begin
  FBorderColor := Value;
  Invalidate;
end;

procedure TButtonGroupEx.SetEdgeButtonColor(const Value: TColor);
begin
  FEdgeButtonColor := Value;
  Invalidate;
end;

procedure TButtonGroupEx.SetGlassButtonColor(const Value: TColor);
begin
  FGlassButtonColor := Value;
  Invalidate;
end;

procedure TButtonGroupEx.SetGlassButtonStyle(const Value: Boolean);
begin
  FGlassButtonStyle := Value;
  Invalidate;
end;

procedure TButtonGroupEx.SetHotButtonColor(const Value: TColor);
begin
  if FHotButtonColor<>Value then begin
    FHotButtonColor:=Value;
    Invalidate;
  end;
end;

procedure TButtonGroupEx.SetOnDrawButton(const Value: TGrpButtonDrawEvent);
begin
  FOnDrawButton := Value;
  Invalidate;
end;

procedure TButtonGroupEx.SetOnDrawIcon(const Value: TGrpButtonDrawIconEvent);
begin
  FOnDrawIcon := Value;
  Invalidate;
end;

procedure TButtonGroupEx.SetRegularButtonColor(const Value: TColor);
begin
  if FRegularButtonColor<>Value then begin
    FRegularButtonColor:=Value;
    Invalidate;
  end;
end;

procedure TButtonGroupEx.SetSelectedButtonColor(const Value: TColor);
begin
  if FSelectedButtonColor<>Value then begin
    FSelectedButtonColor:=Value;
    Invalidate;
  end;
end;

procedure TButtonGroupEx.SetTextIndent(const Value: Byte);
begin
  FTextIndent := Value;
  Invalidate;
end;

procedure TButtonGroupEx.SetWW(const Value: Boolean);
begin
  FWW := Value;
  Invalidate;
end;

end.
