{*******************************************************}
{                                                       }
{          MiTeC MDI Button Group Component             }
{                   version 1.3                         }
{                                                       }
{         Copyright (c) 2007-2012 Michal Mutl           }
{                                                       }
{*******************************************************}

unit MiTeC_MDIButtonGroup;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, CommCtrl, CategoryButtons, ButtonGroup, MiTeC_ButtonGroup, GraphUtil;

const
  cAbout = 'MDI ButtonGroup Control 1.3 - Copyright (c) 2007-2012 Michal Mutl';

type
  PNotifyEvent = ^TNotifyEvent;

  TMDIButtonGroup = class(TButtonGroupEx)
  private
    FAbout: string;
    FActivateList, FDestroyList: TList;
    FOnChange: TNotifyEvent;
    procedure RemoveTab(AChild: TForm);
    procedure ChildActivate(Sender: TObject);
    procedure ChildDestroy(Sender: TObject);
    function GetCaption(index: integer): string;
    function GetCount: integer;
    function GetGlyph(index: integer): integer;
    function GetHint(index: integer): string;
    function GetChild(index: integer): TForm;
    function GetMinimized: integer;
    procedure SetCaption(index: integer; const Value: string);
    procedure SetGlyph(index: integer; const Value: integer);
    procedure SetHint(index: integer; const Value: string);
    function GetActiveChild: TForm;
  protected
    procedure DoItemClicked(const Index: Integer); override;
    procedure Resize; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddButton(AChild: TForm; AImageIndex: Integer; AHint: string);
    procedure MinimizeAll;
    procedure CloseAll;
    procedure DestroyAll;
    function GetChildIndex(AChild: TForm): integer;

    property ActiveMDIChild :TForm read GetActiveChild;
    property MDIChildCount :integer read GetCount;
    property MDIChildren[index :integer] :TForm read GetChild;
    property MinimizedCount :integer read GetMinimized;
    property Captions[index :integer]: string read GetCaption write SetCaption;
    property Glyphs[index :integer]: integer read GetGlyph write SetGlyph;
    property Hints[index :integer]: string read GetHint write SetHint;
  published
    property About: string read FAbout;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MiTeC', [TMDIButtonGroup]);
end;

{ TMDIButtonGroup }

procedure TMDIButtonGroup.AddButton;
var
  i :integer;
  ae,de :PNotifyEvent;
begin
  if Assigned(AChild) then begin
    for i:=0 to Self.Items.Count-1 do
      if TForm(Self.Items[i].Data)=AChild then
        Exit;
    with Self.Items.Add do begin
      Caption:=AChild.Caption;
      Hint:=AHint;
      ImageIndex:=AImageIndex;
      Data:=AChild;
    end;
    if Assigned(AChild.OnActivate) then begin
      new(ae);
      ae^:=AChild.OnActivate;
      FActivateList.add(ae)
    end else
      FActivateList.Add(nil);
    if Assigned(AChild.OnDestroy) then begin
      new(de);
      de^:=AChild.OnDestroy;
      FDestroyList.Add(de);
    end else
      FDestroyList.Add(nil);
    AChild.Hint:=AHint;
    AChild.OnActivate:=ChildActivate;
    AChild.OnDestroy:=ChildDestroy;

    Self.ItemIndex:=Self.Items.Count-1;
    Self.Invalidate;

    ScrollIntoView(Self.ItemIndex);
  end;
end;

procedure TMDIButtonGroup.ChildActivate(Sender: TObject);
var
  i :integer;
begin
  if not(Sender is TMDIButtonGroup) then
    for i:=0 to Self.Items.Count-1 do
      if Self.Items[i].Data=Sender then begin
        Self.ItemIndex:=i;
        Break;
      end;

  if (Self.ItemIndex>-1) then begin
    if Assigned(FActivateList[Self.ItemIndex]) then
      TNotifyEvent(FActivateList[Self.ItemIndex]^)(TForm(Self.Items[Self.ItemIndex].Data));
    ScrollIntoView(Self.ItemIndex);
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TMDIButtonGroup.ChildDestroy(Sender: TObject);
var
  i,idx :integer;
begin
  idx:=-1;
  try
    if Assigned(Sender) and (Sender is TForm) then
      for i:=0 to Self.Items.Count-1 do
        if Assigned(Self.Items[i].Data) and (Self.Items[i].Data=Sender) then begin
          idx:=i;
          Break;
        end;
    if (idx>-1) then begin
      if Assigned(FDestroyList[idx]) then
        TNotifyEvent(FDestroyList[idx]^)(TForm(Self.Items[idx].Data));
      FActivateList.Delete(idx);
      FDestroyList.Delete(idx);
      Self.Items.Delete(idx);
    end;

    Self.ItemIndex:=GetChildIndex(TForm(Owner).ActiveMDIChild);

    if Assigned(FOnChange) then
      FOnChange(Self);
  except
  end;
end;

procedure TMDIButtonGroup.CloseAll;
var
  i,n :integer;
begin
  n:=TForm(Owner).MDIChildCount;
  for i:=n-1 downto 0 do begin
    TForm(Owner).MDIChildren[i].Close;
  end;
end;

constructor TMDIButtonGroup.Create(AOwner: TComponent);
var
  i :integer;
begin
  FAbout:=cAbout;
  if not(AOwner is TCustomForm) then begin
    raise Exception.Create('MDIButtonGroup can be put only on Form.');
    inherited Destroy;
  end else
    if TForm(AOwner).FormStyle<>fsMDIForm then begin
      raise Exception.Create('MDIButtonGroup can be put only on MDIForm.');
      inherited Destroy;
    end;
  for i:=0 to AOwner.ComponentCount-1 do
    if AOwner.Components[i] is TMDIButtonGroup then begin
      raise Exception.Create('Only one MDIButtonGroup can be put on MDIForm.');
      inherited Destroy;
    end;
  inherited;

  FActivateList:=TList.Create;
  FDestroyList:=TList.Create;

  HotButtonColor:=$00EFD3C6;
  SelectedButtonColor:=clInfoBk;
  RegularButtonColor:=GetHighlightColor(clBtnFace, 15);
  Self.ButtonWidth:=100;
  Self.ButtonHeight:=22;
  Self.Height:=22;
  Self.BorderStyle:=bsNone;
  Self.ButtonOptions:=[gboGroupStyle,gboShowCaptions];
  Self.ShowHint:=True;
end;

destructor TMDIButtonGroup.Destroy;
begin
  FActivateList.Free;
  FDestroyList.Free;
  PopupMenu:=nil;
  inherited;
end;

procedure TMDIButtonGroup.DestroyAll;
var
  i,n :integer;
  ca: TCloseAction;
begin
  ca:=caFree;
  n:=TForm(Owner).MDIChildCount;
  for i:=n-1 downto 0 do begin
    if Assigned(TForm(Owner).MDIChildren[i].OnClose) then
      TForm(Owner).MDIChildren[i].OnClose(TForm(Owner).MDIChildren[i],ca);
    TForm(Owner).MDIChildren[i].Free;
  end;
end;

procedure TMDIButtonGroup.DoItemClicked(const Index: Integer);
var
  child :TForm;
begin
  Self.ItemIndex:=Index;
  if Self.ItemIndex>-1 then begin
    Child:=TForm(Self.Items[Self.ItemIndex].Data);
    SendMessage(Child.Handle,WM_NCACTIVATE,WA_ACTIVE,0);
    Child.SetFocus;
    Child.BringToFront;
    if Child.WindowState=wsMinimized then
      Child.WindowState:=wsNormal;
  end;
  inherited;
  UpdateButton(Self.ItemIndex);
  if Assigned(FOnChange) then
      FOnChange(Self);
end;

function TMDIButtonGroup.GetActiveChild: TForm;
begin
  Result:=GetChild(Self.ItemIndex);
end;

function TMDIButtonGroup.GetCaption(index: integer): string;
begin
  if (index>-1) and (index<Self.Items.Count) then
    Result:=Self.Items[index].Caption
  else
    Result:='';
end;

function TMDIButtonGroup.GetChild(index: integer): TForm;
begin
  if (index>-1) and (index<Self.Items.Count) then
    Result:=TForm(Self.Items[index].Data)
  else
    Result:=nil;
end;

function TMDIButtonGroup.GetChildIndex(AChild: TForm): integer;
var
  i: integer;
begin
  Result:=-1;
  for i:=0 to Self.MDIChildCount-1 do
    if Self.MDIChildren[i]=AChild then begin
      Result:=i;
      Break;
    end;
end;

function TMDIButtonGroup.GetCount: integer;
var
  i: Integer;
begin
  Result:=0;
  for i:=0 to Self.Items.Count-1 do
    if Assigned(Self.Items[i].Data) and (TObject(Self.Items[i].Data) is TForm) then
      Inc(Result);
end;

function TMDIButtonGroup.GetGlyph(index: integer): integer;
begin
  if (index>-1) and (index<Self.Items.Count) then
    Result:=Self.Items[index].ImageIndex
  else
    Result:=-1;
end;

function TMDIButtonGroup.GetHint(index: integer): string;
begin
  if (index>-1) and (index<Self.Items.Count) then
    Result:=Self.Items[index].Hint
  else
    Result:='';
end;

function TMDIButtonGroup.GetMinimized: integer;
var
  i :integer;
begin
  Result:=0;
  for i:=0 to Self.Items.Count-1 do
    if GetChild(i).WindowState=wsMinimized then
      Inc(Result);
end;

procedure TMDIButtonGroup.MinimizeAll;
var
  i,n :integer;
begin
  n:=TForm(Owner).MDIChildCount;
  for i:=n-1 downto 0 do
    TForm(Owner).MDIChildren[i].WindowState:=wsMinimized;
  TForm(Owner).ArrangeIcons;
end;

procedure TMDIButtonGroup.RemoveTab(AChild: TForm);
var
  i :integer;
begin
  if Assigned(AChild) then
    for i:=0 to Self.Items.Count-1 do
      if TForm(Self.Items[i])=AChild then begin
        Self.Items.Delete(i);
        FActivateList.Delete(i);
        FDestroyList.Delete(i);
        Break;
      end;
end;

procedure TMDIButtonGroup.Resize;
var
  RowsSeen: Integer;
  ButtonsPerRow: Integer;
  TotalRowsNeeded: Integer;
  BevelHeight: Integer;
begin
  inherited;

  RowsSeen:=CalcRowsSeen;
  ButtonsPerRow:=CalcButtonsPerRow;

  BevelHeight:=2;
  if BorderStyle<>bsNone then
    Inc(BevelHeight,4)
  else
    if BevelKind<>bkNone then begin
      if beTop in BevelEdges then
        Inc(BevelHeight,2);
      if beBottom in BevelEdges then
        Inc(BevelHeight,2);
    end;

  if AutoSize then begin
    Windows.ShowScrollBar(Handle,SB_VERT,False);
    TotalRowsNeeded:=Items.Count div ButtonsPerRow;
    if Items.Count mod ButtonsPerRow<>0 then
      Inc(TotalRowsNeeded);
    Height:=TotalRowsNeeded*ButtonHeight+BevelHeight;
  end else begin
    Height:=ButtonHeight+BevelHeight;
    inherited;
  end;
end;

procedure TMDIButtonGroup.SetCaption(index: integer; const Value: string);
begin
  if (index>-1) and (index<Self.Items.Count) then
    Self.Items[index].Caption:=Value;
end;

procedure TMDIButtonGroup.SetGlyph(index: integer; const Value: integer);
begin
  if (index>-1) and (index<Self.Items.Count) then
    Self.Items[index].ImageIndex:=Value;
end;

procedure TMDIButtonGroup.SetHint(index: integer; const Value: string);
begin
  if (index>-1) and (index<Self.Items.Count) then
    Self.Items[index].Hint:=Value;
end;

end.
